#ifndef _HANDLERS_H
#define _HANDLERS_H

#include "client.h"
#include "user.h"
#include "tester.h"
#include "unknown.h"
#include "admin.h"
#include "config.h"
#include "handler.h"
#include "rrdata.h"
#include "internal_structures.h"
#include "mysqlplus.h"
#include "base64.h"
#include "RRParser.h"

#include <stdlib.h>
#include <queue>
#include <iostream>
#include <strstream>
#include <string.h>
#include <cmath>
#include <search.h>

using namespace MySQLPlus;
using std::queue;
using std::cout;
using std::endl;
using std::basic_string;
using std::istrstream;

extern Configuration config;
extern User users;
extern Tester testers;
extern UNknown unknown;
extern Admin admins;
extern Connection dbCon;
extern Connection dbAdminCon;

void makeCommandHeader(RRData::t_protocol&);
bool isUnknown(long a);
bool isTester(long a);
bool isTeam(long a);
bool isAdmin(long a);
bool isRating(long a);
void parseSemicolonString(set< string >& result, string& string2parse);
bool DBCheck(int res, const RRData& in, RRData& out);
int aux_checkCode(const string&, const string&, string&, int);
int aux_checkPassword(const string&);
int aux_getCheckTasks(queue< t_checkJob >&, int, int);
int aux_checkTask(const string&, int, int);
int aux_checkCompiler(const string&, int);
int aux_checkIfAlreadySolved(const string&, int, int, const string&);
int aux_registerNewTask(const string& TaskNumber, const string& TeamId, const string& ProgramText, int OlympId, int OlympType, unsigned int Penalty, string& DBId, bool tasksCoded, const string& Compiler);
int aux_registerResultOfTesting(const string& DBId, const string& result, const string& testNumber, int OlympId, int OlympType, const string& Task, const string& TeamId, unsigned int Penalty);
int aux_getTestsNumber(const string& Task, int OlympId, int OlympType);
int aux_checkCompilersLists(const set< string >& compilers, int OlympId);
int aux_getTasksWithLessPenalty(unsigned int Penalty, int OlympId, int OlympType, const string& TeamId, const string& Task);
int aux_form211Respose(RRData& rrd_out, int OlympId, int OlympType);
int aux_form212Respose(RRData& rrd_out, int OlympId, int OlympType);
int aux_form203Respose(const RRData& rrd_in, RRData& rrd_out, int OlympId, int OlympType);
int aux_escapeString(string& str);
int aux_remake(int OlympId, int OlympType);
int aux_getLastLogId(int OlympId, int OlympType);
int aux_updateRating(int OlympId, int OlympType, map< int, t_ratingNode >& data);
int aux_getTeams(int OlympId, vector< pair< int, string > >& teams);
int aux_getTasks(int OlympId, int OlympType, vector< int >& tasks);
int aux_getPartOfRating(int OlympId, int OlympType, vector< t_ratingData >& rating, int From, int To = -1);
int aux_disqualifyTeam(int DBId, bool set);

void res_100(const RRData& in, RRData& out);
void res_101(const RRData& in, RRData& out, const string& TaskId);
void res_102(const RRData& in, RRData& out);
void res_200(const RRData& in, RRData& out);
void res_201(const RRData& in, RRData& out);
void res_202(const RRData& in, RRData& out, const string& TaskId, const string& Result, const string& TestNumber);
void res_203(const RRData& in, RRData& out, int OlympId, int OlympType, int& res);
void res_204(const RRData& in, RRData& out);
void res_205(const RRData& in, RRData& out, const char* = NULL);
void res_206(const RRData& in, RRData& out, map< int, t_ratingNode >& data, int lastId = -1);
void res_207(const RRData& in, RRData& out, int From, int To = -1);
void res_208(const RRData& in, RRData& out, int From);
void res_209(const RRData& in, RRData& out);
void res_210(const RRData& in, RRData& out, const string& TaskId);
void res_211(const RRData& in, RRData& out, int OlympId, int OlympType, int& res);
void res_212(const RRData& in, RRData& out, int OlympId, int OlympType, int& res);
void res_214(const RRData& in, RRData& out);
void res_300(const RRData& in, RRData& out, const string& tasks);
void res_301(const RRData& in, RRData& out, const string& Task, const string& Compiler, const string& ProgramText, bool coded = true);
void res_400(const RRData& in, RRData& out, const char* = NULL);
void res_401(const RRData& in, RRData& out);
void res_402(const RRData& in, RRData& out);
void res_403(const RRData& in, RRData& out);
void res_404(const RRData& in, RRData& out, const char* = "Bad syntax of request");
void res_405(const RRData& in, RRData& out);
void res_406(const RRData& in, RRData& out);
void res_407(const RRData& in, RRData& out);
void res_409(const RRData& in, RRData& out);
void res_500(const RRData& in, RRData& out, const char* = NULL);
void res_502(const RRData& in, RRData& out);

void defaultHandler(const RRData& in, RRData& out);

void req_LOGIN(const RRData& in, RRData& out);
void req_LOGOUT(const RRData& in, RRData& out);
void req_TASK(const RRData& in, RRData& out);
void req_READY(const RRData& in, RRData& out);
void req_DONE(const RRData& in, RRData& out);
void req_GET_TASKS(const RRData& in, RRData& out);
void req_GET_COMPILERS(const RRData& in, RRData& out);
void req_GTT(const RRData& in, RRData& out);
void req_START(const RRData& in, RRData& out);
void req_REMAKE(const RRData& in, RRData& out);
void req_STATUS_CHANGE(const RRData& in, RRData& out);
void req_RATING(const RRData& in, RRData& out);
void req_RATING_PART(const RRData& in, RRData& out);
void req_DSQ(const RRData& in, RRData& out);
void req_REACTIVATE(const RRData& in, RRData& out);
void req_INIT(const RRData& in, RRData& out);

void formHandlerMap(t_handlers&);

#endif // _HANDLERS_H