#ifndef _MYSQL_PLUS_H
#define _MYSQL_PLUS_H

#ifdef WIN32
#include <windows.h>
#endif

#include <mysql.h>
#include <string>
#include <vector>
#include <sstream>

namespace MySQLPlus
{
	using std::string;
	using std::vector;
	using std::ostringstream;

	typedef my_ulonglong ULongLong;

	class String;
	class MySQLP;
	class Connection;
	class Query;
	class Result;
	class Row;

	typedef vector< Row > __basic_result;
	typedef vector< String > __basic_row;

	class String : public string 
	{
	public:
		String();
		String( const String & );
		String( const string & );
		String( const char * );
//		int parseInt();
//		unsigned int parseUInt();
//		long parseLong();
//		unsigned long parseULong();
//		long long parseLongLong();
//		float parseFloat();
//		double parseDouble();
//		long double parseLongDouble();
	};

	class MySQLPImplementation
	{
	friend MySQLP;
	friend Connection;
	friend Query;
	protected:
		MySQLPImplementation();
		~MySQLPImplementation();
		MYSQL *_mysql;
		vector< MySQLP * > _family;
	};

	class MySQLP
	{
	public:
		MySQLP();
		MySQLP( const MySQLP & );
		virtual ~MySQLP();
		operator bool (){ return ( _success && _connected ); }
		unsigned int errno() { return mysql_errno( _mysqlimp->_mysql ); }
		string error() { return string( mysql_error( _mysqlimp->_mysql ) ); }
		string escapeString( const string & );
	protected:
		void _register();
		void _unregister();
		MySQLPImplementation *_mysqlimp;
		bool _success;
		bool _connected;
	};

	class Connection : public MySQLP
	{
	public:
		Connection() : MySQLP() {}
		Connection(
			const string &,			//user
			const string & = "",		//password
			const string & = "",		//database
			const string & = "localhost",	//host
			unsigned int = 0);		//port
		Connection( const Connection & );
		~Connection();
		bool connect(
			const string &,			//user
			const string & = "",		//password
			const string & = "",		//database
			const string & = "localhost",	//host
			unsigned int = 0);		//port
		void close();
		bool selectDb( const string & );
		Query query();
	};

	class Query : public MySQLP
	{
	public:
		Query( const MySQLP & );
		Query( const Query & );
		template< class T >
		Query & operator << (const T &o)
		{
			ostringstream oss("");
			oss << o;
			_query += oss.str();
			return *this;
		}
		string preview() { return _query; }
		void reset();
		void clear() { reset(); }
		Result store( const String & /*default_value*/ = "" );
		bool exec();						//0.9.4
		ULongLong insertId();
	protected:
		string _query;
	};

	class Result : protected __basic_result
	{
	friend Query;
	friend Row;
	public:
		Result( const Result & );
		size_t rows() { return _rows; }
		size_t cols() { return _cols; }
		Row operator [] ( long unsigned int );
		Result::iterator begin();
		Result::iterator end();
		string fieldName( long unsigned int );			//0.9.4
		long unsigned int fieldNumber( const string & );	//0.9.4
	protected:
		Result();
		Result( MYSQL_RES *, const String & );
	private:
		size_t _rows, _cols;
		vector< string > _fields;				//0.9.4
		void fixThisInRows();					//0.9.4
	};

	class Row : protected __basic_row
	{
	friend Result;
	public:
		Row( const Row &);
		size_t cols() const { return _res->cols(); }	//0.9.4
		const String& operator [] ( long unsigned int ) const;
		String operator [] ( const string & );			//0.9.4
		string fieldName( long unsigned int i );
		long unsigned int fieldNumber( const string & );
	protected:
		Row();
		Row( Result *, const MYSQL_ROW &, const String & );	//0.9.4
		Result *_res;						//0.9.4
	};
};

#endif
