#include <mysqlplus.h>

using namespace MySQLPlus;

Result::Result()
	: __basic_result()
	, _rows( 0 )
	, _cols( 0 )
{
}

Result::Result( MYSQL_RES *res, const String &default_value )
	: __basic_result()
	, _rows( 0 )
	, _cols( 0 )
{
	_cols = mysql_num_fields( res );
	MYSQL_FIELD *fields = mysql_fetch_fields( res );
	for( long unsigned int i=0; i<_cols; ++i)
		_fields.push_back( fields[i].name );
	MYSQL_ROW api_row;
	while( ( api_row = mysql_fetch_row( res ) ) )
		this->push_back( Row( this, api_row, default_value ) );
	_rows = this->size();
}

Result::Result( const Result &o )
	: __basic_result( o )
	, _rows( o._rows )
	, _cols( o._cols )
	, _fields( o._fields )
{
}

Row Result::operator [] ( long unsigned int i )
{
	if( _rows && static_cast< __basic_result * >(this)->begin()->_res != this ) fixThisInRows();
	return static_cast< __basic_result * >(this)->operator[]( i );
}

Result::iterator Result::begin()
{
	if( _rows && static_cast< __basic_result * >(this)->begin()->_res != this ) fixThisInRows();
	return static_cast< __basic_result * >(this)->begin();
}

Result::iterator Result::end()
{
	if( _rows && static_cast< __basic_result * >(this)->begin()->_res != this ) fixThisInRows();
	return static_cast< __basic_result * >(this)->end();
}

inline void Result::fixThisInRows()
{
	Result::iterator p = static_cast< __basic_result * >(this)->begin();
	while( p != static_cast< __basic_result * >(this)->end() )
	{
		(*p)._res = this;
		++p;
	};
}

string Result::fieldName( long unsigned int i )
{
	return _fields[i];
}

long unsigned int Result::fieldNumber( const string &field )
{
	for( long unsigned int i=0; i<_cols; ++i)
		if( _fields[i] == field ) return i;
	return 0;
}
