/*
	This header file defines Server class, that helps to create application
	which can be service under Windows operating system and daemon under UNIX.
	
	You should derive your own class from class Service and redefine function
	Run() which is pure abstract in base class Service.

	Yo can also redefine virtual functions OnReload() - it is executed when
	server catches signal to reload, and OnTerminate() - it is executed when
	server is catches signal to terminate.

	Your derived class should not declare default constructor. Instead it should
	(and must) define constructor of type:

		<name_of_class>(const char* serverName, int argc, char** argv)

	and should pass all of its arguments to base class's constructor.
	
	After that you should create object of your class in your application
	with parameters:
	1) serverName - service name on Windows OS, on UNIX - ignored
	2) argc - should be passed from main() function
	3) argv - should be passed from main() function

	Then you execute Start() method and everything happens.
*/

#ifndef _PR_SERVER_H
#define _PR_SERVER_H

#include "pr_general.h"

// including headers that we will need
#include <string>
#include <stdio.h>
#ifdef WIN32
#include "windows.h"
static SERVICE_STATUS MyServiceStatus;
static SERVICE_STATUS_HANDLE MyServiceStatusHandle;
// forwarding
class Server;
// global pointer to object Server
static Server* __server = NULL;
void ServiceMain(DWORD, LPTSTR*);
void ServiceCtrlHandler(DWORD);
DWORD ServiceStart(SC_HANDLE);
DWORD ServiceSetNewPath(const char*, SC_HANDLE, SC_HANDLE);
#endif // WIN32

using namespace std;

class Server: public pr{
private:
	Server(){}
protected:
	string ServerName;
	int argc;
	char** argv;
#ifdef WIN32
	void WinService();
	friend void ServiceMain(DWORD, LPTSTR*);
	friend void ServiceCtrlHandler(DWORD);
#endif // WIN32
public:
	Server(const char*, int, char**);
	~Server(){}
	virtual void Run() = 0;
	virtual void OnReload(){}
	virtual void OnTerminate(){}

	bool Start();
	const char* GetServerName();
};

#endif // _PR_SERVER_H