#include "pr_general.h"
#include "pr_socket.h"
#include <string.h>
#include <iostream>

using namespace std;

/*
 Defining Socket methods
*/

Socket::Socket(void):
	sock_fd(0), created(false)
{
	memset((void*)&local, 0, sizeof(struct sockaddr_in));
	memset((void*)&peer, 0, sizeof(struct sockaddr_in));
	sock_fd = socket(AF_INET, SOCK_STREAM, 0);
	if (sock_fd == PR_INVALID_SOCKET){
		sock_fd = 0;
		return;
	}
	created = true;
}

bool Socket::Close(void){
	if (sock_fd != 0){
		if (close(sock_fd) == PR_SOCKET_ERROR){
			return false;
		}
		sock_fd = 0;
	}
	created = false;
	return true;
}

Socket::~Socket(void){
	Close();
}

bool Socket::Created(void){
	return created;
}

bool Socket::makeBlocking(void){
unsigned long a = 0;
#ifdef WIN32
	if (ioctlsocket(sock_fd, FIONBIO, &a) == SOCKET_ERROR){
		return false;
	}
#else
	int cur_flags = fcntl(sock_fd, F_GETFL);
	if (cur_flags == -1){
		return false;
	}
	cur_flags = cur_flags & (~O_NONBLOCK);
	if (fcntl(sock_fd, F_SETFL, cur_flags) == -1){
		return false;
	}
#endif // WIN32
	return true;
}

bool Socket::makeNonBlocking(void){
#ifdef WIN32
	unsigned long a = 1;
	if (ioctlsocket(sock_fd, FIONBIO, &a) == SOCKET_ERROR){
		return false;
	}
#else
	int cur_flags = fcntl(sock_fd, F_GETFL);
	if (cur_flags == -1){
		return false;
	}
	if (fcntl(sock_fd, F_SETFL, cur_flags | O_NONBLOCK) == -1){
		return false;
	}
#endif // WIN32
	return true;
}

bool Socket::wouldBlock(void){
#ifdef WIN32
	return (PR_ERRNO == WSAEWOULDBLOCK);
#else
	return (PR_ERRNO == EWOULDBLOCK);
#endif
}

/*
 Defining stdSocket methods
*/

stdSocket::stdSocket(PR_SOCKET sock, struct sockaddr_in* local_p, struct sockaddr_in* peer_p)
{
	sock_fd = sock;
	created = true;
	memcpy((void*)&local, (const void*)local_p, sizeof(struct sockaddr_in));
	memcpy((void*)&peer, (const void*)peer_p, sizeof(struct sockaddr_in));
}

bool stdSocket::Connect(char* hostname, int Port){
	struct hostent* Host;

	if ((Host = gethostbyname(hostname)) == NULL){
		return false;
	}

	memset((void*)&peer, 0, sizeof(struct sockaddr_in));
	peer.sin_family = AF_INET;
	peer.sin_port = htons(Port);
	memcpy((void*)&(peer.sin_addr), Host->h_addr, sizeof(struct in_addr));

	if (connect(sock_fd, (struct sockaddr*)&peer, sizeof(struct sockaddr)) == PR_SOCKET_ERROR){
		return false;
	}

	return true;
}

char* stdSocket::GetRemoteIp(void){
	return (inet_ntoa(peer.sin_addr));
}

int stdSocket::Recv(char* Buf, int Length){
	return recv(sock_fd, Buf, Length, 0);
}

int stdSocket::Send(char* Mes, int Length){
	return send(sock_fd, Mes, Length, 0);
}


/*
 Defining acceptSocket methods
*/

acceptSocket::acceptSocket(int Port):
	Socket()
{
	if (!Created()) return;
	local.sin_family = AF_INET;
	local.sin_port = htons(Port);
#ifdef WIN32
	local.sin_addr.S_un.S_addr = INADDR_ANY;
#else
	local.sin_addr.s_addr = INADDR_ANY;
#endif

	if (bind(sock_fd, (struct sockaddr*)&local, sizeof(struct sockaddr)) == PR_SOCKET_ERROR){
		created = false;
		return;
	}

	if (listen(sock_fd, 5) == PR_SOCKET_ERROR){
		created = false;
		return;
	}
}

stdSocket* acceptSocket::Accept(void){
#ifdef WIN32
	int size;
#else
	unsigned int size;
#endif // WIN32
	size = sizeof(struct sockaddr_in);
	memset((void*)&peer, 0, sizeof(struct sockaddr_in));
	PR_SOCKET accepted_fd = accept(sock_fd, (struct sockaddr*)&peer, &size);
	if (accepted_fd == PR_INVALID_SOCKET)
		return NULL;
	return new stdSocket(accepted_fd, &local, &peer);
}
