//Buffer.h - header file for CBuffer class
#ifndef _BUFFER_H
#define _BUFFER_H

#include <string>
#include "pr_socket.h"
#define BUFFER_SIZE 1048576
#define STORY_SIZE 10

typedef unsigned long ulong;

class CBuffer
{
public:
	CBuffer(stdSocket* _socket,ulong _wbsize);
	
	char getch(void);
	ulong getline(char* _string,ulong _size,char _delim = '\n');
	ulong getline(std::string& _str,char _delim = '\n');
	ulong safegetline(char* _string,ulong _size,char _delim = '\n');
	ulong safegetline(std::string& _str,char _delim = '\n');
	ulong getword(char* _word,ulong _size);
	ulong getword(std::string& _std);
	ulong safegetword(char* _word,ulong _size);
	ulong safegetword(std::string& _std);
	ulong read(char* _buff,ulong _size,ulong _read);
	ulong read(std::string& _std,ulong _read);

	bool is_empty(void);
	bool is_online(void);
	bool has_error(void);

	bool rollback(ulong _n);
	void free(void);
	void clear(void);
	int getsize(void);

	~CBuffer();

protected:
	struct SBufferList
	{
		char* buffer;
		ulong size;
		SBufferList* next;
		SBufferList* prev;
	};
	struct SPointer
	{
		SBufferList* buffer;
		ulong poiner;
		SPointer* next;
	};
	stdSocket* socket;
	SBufferList* buffer_top;
	SBufferList* buffer_bottom;
	SPointer* pointer;
	ulong size;
	bool online;
	bool empty;
	bool error;

private:
	void read_socket(void);
	void erase(SPointer* _pointer);
	void next(SBufferList*& _buffer,ulong& _pointer);
	void addmark(SBufferList* _buffer,ulong _pointer);
	char* work_buffer;
	ulong wbsize;
};

#endif // _BUFFER_H