#ifndef _HANDLER_H
#define _HANDLER_H

#include <map>
#include <string>
#include <string.h>
#include "rrdata.h"

#define MAKE_HANDLER_TYPE(state, type) ((state << 8) | type)

using std::map;
using std::string;
using std::pair;

// Handler function itself
typedef void (*t_handlerFunction)(const RRData&, RRData&);
// Pair of handler function and its type: 1 - without body, 2 - with body
typedef pair< unsigned short, t_handlerFunction > t_handler;
// map of handlers
typedef map< string, t_handler > t_handlers;

// class Handler that incapsulates handlers map and realizes its versioning
class Handler{
public:
	Handler(t_handlers* _handlers,	// Map of handlers
			t_handlerFunction _defaultHandler,	// Default handler
			string _protocolName,	// Name of the current protocol
			int _major,	// Major number of the current ptorocol's version
			int _minor, // Minor number of the current ptorocol's version
			Handler* _base = NULL);	// Pointer to Handler class of preceding version

	enum t_handlerState { derived, recent, deprecated };
	enum t_handlerType { nocommand, withoutbody, withbody };
// returns command's type
	t_handlerType getCommandType(string command);
// handles in structure and generate out structure
	void Handle(RRData& in, RRData& out);

private:
	t_handlers* handlers;
	t_handlerFunction defaultHandler;
	Handler* base;
	struct RRData::t_protocol protocol;

	Handler(){}
	inline t_handlerState getState(unsigned short int a) { return (t_handlerState)(a >> 8); }
	inline t_handlerType getType(unsigned short int a) { return (t_handlerType)(a & 255); }
};

#endif // _HANDLER_H