// Buffer.cpp - implementation of CBuffer class
#include "Buffer.h"
#include <string.h>
#include <iostream>

using namespace std;

CBuffer::CBuffer(stdSocket* _socket,ulong _wbsize=BUFFER_SIZE)
{
	socket=_socket;
	wbsize=_wbsize;
	work_buffer=new char[wbsize+1];
	online=empty=true;
	error=false;
	buffer_top=buffer_bottom=NULL;
	
	pointer=new SPointer;
	pointer->buffer=buffer_top;
	pointer->next=NULL;
	pointer->poiner=0;
	
	read_socket();

	if(!socket->Created())
		online = false;
};

ulong CBuffer::getline(char* _string,ulong _size,char _delim)
{
	string _str;
	getline(_str,_delim);
	return _str.copy(_string,_size);
};


ulong CBuffer::getline(string& _str,char _delim)
{
	read_socket();
	
	if(pointer->buffer==NULL)
	{
		error=true;
		return 0;
	};
	SBufferList* _curr_buf=pointer->buffer;
	ulong _curr_pos=pointer->poiner;
	_str.erase();
	while((_curr_buf->buffer)[_curr_pos]!=_delim)
	{
		_str+=(_curr_buf->buffer)[_curr_pos];
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL)
			break;
	};
	next(_curr_buf,_curr_pos);
	addmark(_curr_buf,_curr_pos);

	error=false;

	return (ulong)_str.size();
};

ulong CBuffer::safegetline(char* _string,ulong _size,char _delim)
{
	string _str;
	safegetline(_str,_delim);
	return _str.copy(_string,_size);
};

ulong CBuffer::safegetline(string& _str,char _delim)
{	
	read_socket();

	if(pointer->buffer==NULL)
	{
		error=true;
		return 0;
	};

	SBufferList* _curr_buf=pointer->buffer;
	ulong _curr_pos=pointer->poiner;
	while((_curr_buf->buffer)[_curr_pos]!=_delim)
	{
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL) break;
	};

	if(_curr_buf==NULL)
	{
		error=true;
		return 0;
	};

	_curr_buf=pointer->buffer;
	_curr_pos=pointer->poiner;
	_str.erase();
	while((_curr_buf->buffer)[_curr_pos]!=_delim)
	{
		_str+=(_curr_buf->buffer)[_curr_pos];
		next(_curr_buf,_curr_pos);
	};

	next(_curr_buf,_curr_pos);
	addmark(_curr_buf,_curr_pos);

	error=false;
	
	return _str.size();
};

char CBuffer::getch(void)
{
	read_socket();

	if(empty)
		return '\0';
	char _c=(pointer->buffer->buffer)[pointer->poiner];
	addmark(pointer->buffer,pointer->poiner);
	next(pointer->buffer,pointer->poiner);

	return _c;
};

ulong CBuffer::getword(char* _word,ulong _size)
{
	string _str;
	getword(_str);
	return _str.copy(_word,_size);
};

ulong CBuffer::getword(string& _str)
{
	read_socket();

	if(pointer->buffer==NULL)
	{
		error=true;
		return 0;
	};
	SBufferList* _curr_buf=pointer->buffer;
	ulong _curr_pos=pointer->poiner;
	while(isspace((_curr_buf->buffer)[_curr_pos]))
	{
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL)
			break;
	};
	if(_curr_buf==NULL)
	{
		error=true;
		return 0;
	};
	_str.erase();
	while(!isspace((_curr_buf->buffer)[_curr_pos]))
	{
		_str+=(_curr_buf->buffer)[_curr_pos];
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL)
			break;
	};
	addmark(_curr_buf,_curr_pos);

	error=false;

	return _str.size();
};

ulong CBuffer::safegetword(char* _word,ulong _size)
{
	string _str;
	safegetword(_str);
	return _str.copy(_word,_size);
};

ulong CBuffer::safegetword(string& _str)
{
	read_socket();

	if(pointer->buffer==NULL)
	{
		error=true;
		return 0;
	};
	SBufferList* _curr_buf=pointer->buffer;
	ulong _curr_pos=pointer->poiner;
	while(isspace((_curr_buf->buffer)[_curr_pos]))
	{
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL)
			break;
	};
	if(_curr_buf==NULL)
	{
		error=true;
		return 0;
	};
	SBufferList* _buf=_curr_buf;
	ulong _position=_curr_pos;
	while(!isspace((_curr_buf->buffer)[_curr_pos]))
	{
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL)
			break;
	};
	if(_curr_buf==NULL)
	{
		error=true;
		return 0;
	};
	_curr_buf=_buf;
	_curr_pos=_position;
	_str.erase();
	while(!isspace((_curr_buf->buffer)[_curr_pos]))
	{
		_str+=(_curr_buf->buffer)[_curr_pos];
		next(_curr_buf,_curr_pos);
	};
	addmark(_curr_buf,_curr_pos);

	error=false;

	return _str.size();
};

ulong CBuffer::read(char* _buff,ulong _size,ulong _read)
{
	string _str;
	read(_str,_read);
	return _str.copy(_buff,_size);
};

ulong CBuffer::read(string& _str,ulong _read)
{
	read_socket();

	SBufferList* _curr_buf=pointer->buffer;
	ulong _curr_pos=pointer->poiner;
	ulong _curr_read=0;
	while(_curr_read<_read)
	{
		_curr_read++;
		next(_curr_buf,_curr_pos);
		if(_curr_buf==NULL)
			break;
	};
	if(_curr_read!=_read)
	{
		error=true;
		return 0;
	};
	_curr_buf=pointer->buffer;
	_curr_pos=pointer->poiner;
	_curr_read=0;
	_str.erase();
	while(_curr_read<_read)
	{
		_str+=(_curr_buf->buffer)[_curr_pos];
		next(_curr_buf,_curr_pos);
		_curr_read++;
	};
	addmark(_curr_buf,_curr_pos);

	error=false;

	return _str.size();
};

bool CBuffer::is_empty(void)
{
	return empty;
};

bool CBuffer::is_online(void)
{
	return online;
};

bool CBuffer::has_error(void)
{
	return error;
};

bool CBuffer::rollback(ulong _n)
{
	SPointer* _p=pointer;
	while(_p->next!=NULL && _n>0)
	{
		pointer=_p->next;
		delete _p;
		_p=pointer;
		_n--;
	};
	if (_n>0)
		return false;
	return true;
};

void CBuffer::free(void)
{
	erase(pointer);
}

void CBuffer::read_socket(void)
{
	int _rec;
	while((_rec=socket->Recv(work_buffer,wbsize))>0)
	{
		SBufferList* _bl=new SBufferList;
		_bl->next=_bl->prev=NULL;
		_bl->size=_rec;
		_bl->buffer=new char[_rec];
		memcpy(_bl->buffer,work_buffer,_rec);

		if(buffer_bottom==NULL)
		{
			buffer_top=buffer_bottom=_bl;
			size=_bl->size;
			empty=false;
		}
		else
		{
			buffer_bottom->next=_bl;
			_bl->prev=buffer_bottom;
			_bl->next=NULL;
			size+=_bl->size;
			buffer_bottom=_bl;
		};

		if(pointer->buffer==NULL)
		{
			pointer->buffer=buffer_bottom;
			pointer->poiner=0;
		}
	};
	if(!(_rec == PR_SOCKET_ERROR && socket->wouldBlock()))
	{
		online=false;
		error=true;
	}
};

// Erase all save points before _pointer
void CBuffer::erase(SPointer* _pointer)
{
	if(_pointer==NULL)
		return;
	SBufferList* _bl=buffer_top;
	if(_pointer->buffer=NULL)
	{
		while(_bl!=NULL)
		{
			buffer_top=buffer_top->next;
			size-=_bl->size;
			delete _bl->buffer;
			delete _bl;
			_bl=buffer_top;
		};
		buffer_bottom=NULL;
	}
	else
	{
		while(_bl!=_pointer->buffer)
		{
			buffer_top=buffer_top->next;
			size-=_bl->size;
			delete _bl->buffer;
			delete _bl;
			_bl=buffer_top;
		};
		_bl->prev=NULL;
		buffer_top=_bl;
	};
	SPointer* _p=_pointer->next;
	_pointer->next=NULL;
	while(_p!=NULL)
	{
		SPointer* _next=_p->next;
		delete _p;
		_p=_next;
	};
};

void CBuffer::next(SBufferList*& _buffer,ulong& _pointer)
{
	if(_buffer==NULL)
		return;
	if(_pointer==_buffer->size)
	{
		_buffer=_buffer->next;
		_pointer=0;
	}
	else
	{
		_pointer++;
	};
	if(_buffer==NULL)
		empty=true;
}

void CBuffer::addmark(SBufferList* _buffer,ulong _pointer)
{
	SPointer* _p=new SPointer;
	_p->buffer=_buffer;
	_p->poiner=_pointer;
	_p->next=pointer;
	pointer=_p;

	int _i=STORY_SIZE+1;
	while(_i>0)
	{
		if(_p==NULL) break;
		_p=_p->next;
		_i--;
	};
	if(_i==0)
		erase(_p);
}

void CBuffer::clear(void)
{
	if(!empty)
		addmark(NULL,0);
	erase(pointer);
}

int CBuffer::getsize(void)
{
	return size;
}

CBuffer::~CBuffer()
{
	clear();
	delete pointer;
	delete[] work_buffer;
}