#include "protocol.h"
#include <strstream>
#include <sstream>
#include <iostream>

using std::ostrstream;
using std::ostringstream;
using std::cout;
using std::ends;

Protocol::Protocol(stdSocket* sock, Handler* handler): parser_(NULL){
	socket_ = sock;
	handler_ = handler;
	buffer_ = new CBuffer(sock, 1048576);
}

Protocol::~Protocol(){
	delete buffer_;
	//socket_->Close();
	//delete socket_;
}

bool Protocol::Parse(RRData& in){
	if (parser_ != NULL){
		return parser_->Parse(&in);
	}
	return false;
}

bool Protocol::Online(void){
	return buffer_->is_online();
}

bool Protocol::PartialExecution(void){
	if (parser_->Error() == RRParser::rrp_part_body ||
		parser_->Error() == RRParser::rrp_part_command ||
		parser_->Error() == RRParser::rrp_part_header)
		return true;
	return false;
}

int Protocol::calculateSize(RRData& a){
	int len = 20;
	len += a.protocol.name.size();
	len += a.command.name.size();
	len += a.command.parameter.size();
	
	RRData::t_headers::iterator e = a.headers.end();
	RRData::t_headers::iterator i = a.headers.begin();
	while (i != e){
		len += i->first.size();
		len += i->second.size();
		len += 3;
		i++;
	}
	
	len += a.body.size();
	return len;
}

//////////////////////////////////////////////////////
// Client Protocol
//////////////////////////////////////////////////////

ClientProtocol::ClientProtocol(stdSocket* sock, Handler* handler):
Protocol(sock, handler)
{
	parser_ = new RRClientParser(*buffer_, *handler_);
}

ClientProtocol::~ClientProtocol(){
	delete parser_;
}

bool ClientProtocol::Generate(RRData& out){
	int len = calculateSize(out);
	
	char* buf = new char[len];
	if (buf == NULL)
		return false;
	
	ostrstream o(buf, len);
	
	o << out.command << ' ' << out.protocol << '\n';
	o << out << ends;

	socket_->Send(buf, (int)o.tellp() - 1);
	delete[] buf;

	return true;

}

bool ClientProtocol::Handle(RRData::t_params params_){
	// something wrong with buffer
	if (parser_->Error() == RRParser::rrp_buffer_error){
		return false;
	}

	// read buffer and form input RRData structure
	if (Parse(in)){
		// no errors - request was fully parsed
		//cout << "Good request read" << endl;
		handler_->Handle(in, out);
		return true;
	} else {
		if (parser_->Error() == RRParser::rrp_request_error){
			// some errors in request
			handler_->Handle(in, out);
			//cout << "Bad request read" << in.command << endl;
			return true;
		} else {
			// request was partly parsed
			//cout << "Part of request read" << in.command << endl;
			return false;
		}
	}
}


//////////////////////////////////////////////////////
// Server Protocol
//////////////////////////////////////////////////////

ServerProtocol::ServerProtocol(stdSocket* sock, Handler* handler):
Protocol(sock, handler)
{
	parser_ = new RRServerParser(*buffer_, *handler_);
}

ServerProtocol::~ServerProtocol(){
	delete parser_;
}

bool ServerProtocol::Generate(RRData& out){
	int len = calculateSize(out);
	
	char* buf = new char[len];
	if (buf == NULL)
		return false;
	
	ostrstream o(buf, len);
	
	o << out.protocol << ' ' << out.command << '\n';
	o << out << ends;

	socket_->Send(buf, (int)o.tellp() - 1);
	//cout << "Generated response: " << out.command.name.c_str() << endl;
	delete[] buf;

	/*
	cout << "Response length: " << (int)o.tellp() << endl;
	cout << "Response: ";
	cout.write(buf, (int)o.tellp());
	cout << endl;
	*/

	return true;
}

bool ServerProtocol::Handle(RRData::t_params params_){
	// something wrong with buffer
	if (parser_->Error() == RRParser::rrp_buffer_error){
		return false;
	}

	// read buffer and form input RRData structure
	in.params = params_;
	out.params = params_;
	if (Parse(in)){
		// no errors - request was fully parsed
		handler_->Handle(in, out);
		Generate(out);
		//cout << "Good request" << endl;
		return true;
	} else {
		if (parser_->Error() == RRParser::rrp_request_error){
			// some errors in request
			handler_->Handle(in, out);
			Generate(out);
			//cout << "Bad request" << endl;
			return true;
		} else {
			// request was partly parsed
			switch (parser_->Error()){
				case RRParser::rrp_part_command:/* cout << "Part command" << endl;*/ break;
				case RRParser::rrp_part_header: /*cout << "Part header" << endl;*/ break;
				case RRParser::rrp_part_body: /*cout << "Part body" << endl;*/ break;
			}
			return false;
		}
	}
}