#include "handlers.h"

//////////////////////////////////////////////////////////////////
// Auxiliary functions
//////////////////////////////////////////////////////////////////

void makeCommandHeader(RRData::t_protocol& a){
	a.name = "OLYMPIA";
	a.version.major = 1;
	a.version.minor = 0;
}

bool isUnknown(long a){
	if (a == Client::t_admin || a == Client::t_rating || a == Client::t_tester || a == Client::t_user)
		return false;

	return true;
}

bool isTester(long a){
	return (a == Client::t_tester);
}

bool isTeam(long a){
	return (a == Client::t_user);
}

bool isAdmin(long a){
	return (a == Client::t_admin);
}

bool isRating(long a){
	return (a == Client::t_rating);
}

void parseSemicolonString(set< string >& result, string& string2parse){
	if (string2parse.empty()) return;

	const basic_string< char >::size_type npos = -1;
	basic_string< char >::size_type indexCh, indexCh2;
	indexCh2 = 0;
	
	indexCh = string2parse.find_first_of(";", indexCh2 + 1);
	string token;

	while (indexCh != npos){
		token.assign(string2parse, indexCh2, indexCh - indexCh2);
		result.insert(token);
		indexCh2 = indexCh + 1;
		indexCh = string2parse.find_first_of(";", indexCh2);
	}
	token.assign(string2parse, indexCh2, indexCh - indexCh2);
	result.insert(token);
}

void res_500(const RRData& in, RRData& out, const char*);
bool DBCheck(int res, const RRData& in, RRData& out){
	if (res == -1){
		res_500(in, out, "Database is not available");
		return false;
	}
	if (res == -2){
		res_500(in, out, "Database error");
		return false;
	}
	if (res == -3){
		res_500(in, out, "Internal error");
		return false;
	}
	
	return true;
}

int aux_checkCode(const string& Code, const string& IP, string& dbId, int OlympId){
	if (!dbCon) return -1;
	
	Query q = dbCon;
	q << "SELECT a.TeamId as TeamId, a.Status as Status FROM team as a, ip as b WHERE a.OlympId=";
	q << OlympId;
	q << " AND a.IPId=b.IPId AND b.ip=\'";
	q << IP.c_str() << "\' AND a.Code=\'";
	q << Code.c_str() << "\'";

	Result res = q.store("NULL");
	if (!q)
		return -2;
	if (res.rows() == 0) return 0;

	if (res[0]["Status"] == "D") return -3;
	dbId = res[0]["TeamId"];
	return 1;
}

int aux_checkPassword(const string& Password){
	if (!dbCon) return -1;
	
	Query q = dbAdminCon;
	q << "SELECT Host FROM user WHERE ";
	q << "password=PASSWORD(\'" << Password.c_str() << "\') AND User=";
	q << "\'olympiadmin\'";

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	return 1;
}

int aux_getCheckTasks(queue< t_checkJob >& checkJobs, int OlympId, int OlympType){
	if (!dbCon) return -1;
	
	Query q = dbCon;
	q << "SELECT a.TaskNumber as TaskNumber, a.ProgrammText as ProgrammText, a.ExpectedResult as ExpectedResult, a.ExpectedFailedTest as ExpectedFailedTest, a.CompilerId as CompilerId FROM standard_task_check as a, task as b WHERE a.OlympId=";
	q << OlympId;
	q << " AND a.OlympType=";
	q << OlympType;
	q << " AND a.OlympId=b.OlympId AND a.OlympType=b.OlympType AND a.TaskNumber=b.TaskNumber";

	Result res = q.store("NULL");
	if (!q)	return -2;
	if (res.rows() == 0) return 1;

	t_checkJob checkJob;
	for (unsigned int i = 0; i < res.rows(); i++){
		checkJob.Compiler = res[i]["CompilerId"];
		checkJob.expectedResult.result = res[i]["ExpectedResult"];
		checkJob.expectedResult.testNumber = res[i]["ExpectedFailedTest"];
		checkJob.ProgramText = res[i]["ProgrammText"];
		checkJob.Task = res[i]["TaskNumber"];
		checkJobs.push(checkJob);
	}
	return 1;
}

int aux_checkTask(const string& TaskId, int OlympId, int OlympType){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT TaskNumber FROM task WHERE OlympId=";
	q << OlympId;
	q << " AND OlympType=";
	q << OlympType;
	q << " AND TaskNumber=\'";
	q << TaskId.c_str();
	q << "\'";

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	return 1;
}

int aux_checkCompiler(const string& CompilerId, int OlympId){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT CompilerId FROM olymp_compiler WHERE OlympId=";
	q << OlympId;
	q << " AND CompilerId=\'";
	q << CompilerId.c_str() << "\'";

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	return 1;
}

int aux_checkIfAlreadySolved(const string& TaskId, int OlympId, int OlympType, const string& TeamId){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT Result FROM rating WHERE OlympId=";
	q << OlympId;
	q << " AND OlympType=";
	q << OlympType;
	q << " AND TaskNumber=\'";
	q << TaskId.c_str();
	q << "\' AND TeamId=\'";
	q << TeamId.c_str();
	q << "\' AND Result=\'+\'";

	Result res = q.store("NULL");
	if (!q) return -2;

	if (res.rows() == 0) return 0;

	return 1;
}

int aux_registerNewTask(const string& TaskNumber, const string& TeamId, const string& ProgramText, int OlympId, int OlympType, unsigned int Penalty, string& DBId, bool tasksCoded, const string& Compiler){
	if (!dbCon) return -1;

	string programText;

	if ((tasksCoded == true) && (ProgramText.size() > 0)){
		// tasks is coded with Base64. We should decode it before storing in database
		
		int buf_size = (int)ProgramText.size() + 2;
		
		char* bufCoded = new char[buf_size];
		char* bufPlain = new char[buf_size];
		strcpy(bufPlain, ProgramText.c_str());
		char* i = bufPlain;
		char* j = bufCoded;
		while (1){
			if (*i == '\0')
				break;
			if ((unsigned int)(i - bufPlain) >= ProgramText.size())
				break;
			if (*i == '\r' || *i == '\n'){
				i++;
				continue;
			}
			*j++ = *i++;
		}
		*j = '\0';

		int res = base64_decode_binary((unsigned char*)bufPlain, (const char*)bufCoded);
		
		bufPlain[res] = '\0';

		programText = bufPlain;
		delete[] bufCoded;
		delete[] bufPlain;
	} else {
		programText = ProgramText;
	}

	Query q = dbCon;
	q << "INSERT INTO log VALUES(\'\', ";
	q << OlympId << ", " << OlympType << ", \'" << TaskNumber.c_str() << "\', \'";
	q << TeamId.c_str() << "\', -1, 0, \'" << q.escapeString(programText).c_str() << "\', \'" << Compiler.c_str() << "\', " << Penalty << ")";
	

	if (!q.exec()) return -2;

	q.clear();
	q << "SELECT last_insert_id() as id FROM log";
	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	DBId = res[0]["id"];

	return 1;
}

int aux_registerResultOfTesting(const string& DBId, const string& result, const string& testNumber, int OlympId, int OlympType, const string& Task, const string& TeamId, unsigned int Penalty){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "UPDATE log SET Result=\'";
	q << result.c_str() << "\', FailedTest=\'" << testNumber.c_str() << "\'";
	q << " WHERE LogEntryId=\'" << DBId.c_str() << "\'";

	if (!q.exec()) return -2;

	q.clear();

	q << "SELECT Result, FailsNumber, Penalty "
		 "FROM rating "
		 "WHERE OlympId=" << OlympId << " "
			"AND OlympType=" << OlympType << " "
			"AND TaskNumber=\'" << Task.c_str() << "\' "
			"AND TeamId=\'" << TeamId.c_str() << "\'";

	Result res = q.store("NULL");
	if (!q) return -2;

	if (res.rows() == 0){
		q.clear();
		q << "INSERT INTO rating VALUES(" << OlympId << ", " << OlympType << ", ";
		q << "\'" << Task.c_str() << "\', \'" << TeamId.c_str() << "\', ";
		if (result.c_str()[0] == '0')
			q << "\'+\', 0, " << Penalty << ")";
		else
			q << "\'-\', 1, " << Penalty << ")";
		if (q.exec())
			return 1;
		else
			return -2;
	}

	q.clear();
	unsigned int oldPenalty = atoi(res[0]["Penalty"].c_str());
	unsigned int oldFailsNumber = atoi(res[0]["FailsNumber"].c_str());
	int num;
	if (res[0]["Result"][0] == '-'){
		if (result.c_str()[0] != '0'){
			q << "UPDATE rating SET FailsNumber=" << (oldFailsNumber + 1);
			if (oldPenalty < Penalty){
				q << ", Penalty=" << Penalty;
			}
			q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType << " AND TaskNumber=";
			q << "\'" << Task.c_str() << "\' AND TeamId=\'" << TeamId.c_str() << "\'";
			if (!q.exec()) return -2;
			return 1;
		} else {
			if (oldPenalty <= Penalty){
				q << "UPDATE rating SET Penalty=" << Penalty << ", Result=\'+\'";
				q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType << " AND TaskNumber=";
				q << "\'" << Task.c_str() << "\' AND TeamId=\'" << TeamId.c_str() << "\'";
				if (!q.exec()) return -2;
				return 1;
			} else {
				num = aux_getTasksWithLessPenalty(Penalty, OlympId, OlympType, TeamId, Task);
				if (num == -2 || num == -1 || num == 0) return num;
				q << "UPDATE rating SET Penalty=" << Penalty << ", Result=\'+\', FailsNumber=" << num;
				q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType << " AND TaskNumber=";
				q << "\'" << Task.c_str() << "\' AND TeamId=\'" << TeamId.c_str() << "\'";
				if (!q.exec()) return -2;
				return 1;
			}
		}
	} else {
		if (result.c_str()[0] != '0'){
			if (oldPenalty >= Penalty){
				q << "UPDATE rating SET FailsNumber=" << (oldFailsNumber + 1);
				q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType << " AND TaskNumber=";
				q << "\'" << Task.c_str() << "\' AND TeamId=\'" << TeamId.c_str() << "\'";
				if (!q.exec()) return -2;
				return 1;
			}
		}
		else {
			if (oldPenalty >= Penalty){
				num = aux_getTasksWithLessPenalty(Penalty, OlympId, OlympType, TeamId, Task);
				if (num == -2 || num == -1) return num;
				q << "UPDATE rating SET Penalty=" << Penalty << ", FailsNumber=" << num;
				q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType << " AND TaskNumber=";
				q << "\'" << Task.c_str() << "\' AND TeamId=\'" << TeamId.c_str() << "\'";
				if (!q.exec()) return -2;
				return 1;
			}
		}
	}

	return 1;
}

int aux_getTestsNumber(const string& Task, int OlympId, int OlympType){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT count(TestNumber) as num FROM test";
	q << " WHERE OlympId=";
	q << OlympId;
	q << " AND OlympType=";
	q << OlympType;
	q << " AND TaskNumber=\'";
	q << Task.c_str() << "\'";

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return -2;

	int num = atoi(res[0]["num"].c_str());

	return num;
}

int aux_checkCompilersLists(const set< string >& compilers, int OlympId){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT a.CompilerId as CompilerId FROM compiler as a, olymp_compiler as b";
	q << " WHERE b.OlympId=";
	q << OlympId;
	q << " AND a.CompilerId=b.CompilerId";

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	for (unsigned int i = 0; i < res.rows(); i++){
		if (compilers.find(res[i]["CompilerId"]) == compilers.end()) return 0;
	}

	return 1;
}

int aux_getTasksWithLessPenalty(unsigned int Penalty, int OlympId, int OlympType, const string& TeamId, const string& Task){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT COUNT(LogEntryId) as num FROM log";
	q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType << " AND TaskNumber=";
	q << "\'" << Task.c_str() << "\' AND TeamId=\'" << TeamId.c_str() << "\' AND Penalty<" << Penalty;

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	int num = atoi(res[0]["num"].c_str());

	return num;
}

int aux_form211Respose(RRData& rrd_out, int OlympId, int OlympType){
	if (!dbCon)
		return -1;

	Query query(dbCon);
	query << 
		" select TaskNumber, Name "
		" from task "
		" where OlympId = " << OlympId << " " <<
		"       and OlympType = " << OlympType;

	Result res(query.store());

	if (!query) return 0;
	if (res.rows() == 0) return -2;

	char buf[64];
	rrd_out.headers["Tasks-Number"] = itoa((int)res.rows(), buf, 10);
	Result::const_iterator i = res.begin();
	Result::const_iterator e = res.end();
	string body;
	for(; i != e; ++i)
		body += (*i)[0] + "\t" + (*i)[1] + "\n";

	int a = (int)body.size();
	char* b = new char[20];
	b = itoa((int)body.size(), b, 10);
	rrd_out.headers["Content-Length"] = itoa((int)body.size(), buf, 10);
	rrd_out.body.swap(body);

	return 1;
}

int aux_form212Respose(RRData& rrd_out, int OlympId, int OlympType){
	if (!dbCon)
		return -1;

	Query query(dbCon);
	query <<
		" select c.CompilerId, c.Name "
		" from compiler as c, olymp_compiler as oc "
		" where c.CompilerId = oc.CompilerId "
		"       and oc.OlympId = " << OlympId;

	Result res = query.store();

	if (!query) return 0;
	if (res.rows() == 0) return -2;

	char buf[64];
	rrd_out.headers["Compilers-Number"] = itoa((int)res.rows(), buf, 10);
	Result::const_iterator i = res.begin();
	Result::const_iterator e = res.end();
	string body;
	for(; i != e; ++i)
		body += (*i)[0] + "\t" + (*i)[1] + "\n";

	rrd_out.headers["Content-Length"] = itoa((int)body.size(), buf, 10);
	rrd_out.body.swap(body);
	
	return 1;
}

int aux_form203Respose(const RRData& rrd_in, RRData& rrd_out, int OlympId, int OlympType){
	class TextHelper {
		string& text_;
	public:
		TextHelper(string& text): text_(text) {}
		static string CodedBase64StringEx(const string& strToCode) {
			if (strToCode.size() == 0) return "";
			int buf_size;
			if (strToCode.size() < 50)
				buf_size = 100;
			else
				buf_size = (int)ceil(1.5 * strToCode.size());
			char* buf = new char[buf_size];
			int res = base64_encode_binary(buf, (const unsigned char*)strToCode.data(), (int)strToCode.size());
			
			buf[res] = '\0';
			
			string ret = buf;
			delete[] buf;
			return ret;
		}

		void AddStringToText(const string& strToAdd = "") {
			text_ += strToAdd + "\n";
		}
		void AddHeaderToText(const string& headerKey, const string& headerValue) {
			AddStringToText(headerKey + ": " + headerValue);
		}
		void AddHeaderToText(const string& headerKey, size_t headerValue) {
			char buf[64];
			AddHeaderToText(headerKey, itoa((int)headerValue, buf, 10));
		}
		static bool NotNull(const string& value) {

			return !value.empty(); // ???
		}
	};

	class Helper {
	public:
		static set< string > ParseNeededTasks(const RRData& rrd) {
			set< string > ret;
			RRData::t_headers::const_iterator hi = rrd.headers.find("Tasks");
			if(hi != rrd.headers.end()) {
				string::const_iterator si = hi->second.begin();
				string::const_iterator se = hi->second.end();
				string strNum;
				while(si != se) {
					while(si != se && isspace(*si)) ++si;
					strNum = "";
					while(si != se && isdigit(*si)) {
						strNum += *si;
						++si;
					}
					if(strNum.empty())
						return ret;
					while(si != se && isspace(*si)) ++si;
					if(si == se) {
						ret.insert(strNum);
						return ret;
					}
					if(*si != ',')
						return ret;
					++si;
					ret.insert(strNum);
				}
			}
			return ret;
		}
		static string ParseTaskString(const set< string >& tasks) {
			string ret;
			if(!tasks.empty()) {
				set< string >::const_iterator i = tasks.begin();
				set< string >::const_iterator e = tasks.end();
				--e;
				for(; i != e; ++i) 
					ret += *i + ",";
				ret += *i;
			}
			return ret;
		}
		static string ParseStdHeaderString(const map< string, string >& tasks)
		{
			string ret;
			if(!tasks.empty()) {
				map< string, string >::const_iterator i = tasks.begin();
				map< string, string >::const_iterator e = tasks.end();
				--e;
				for(; i != e; ++i)
					ret += i->first + "-" + i->second + ";";
				ret += i->first + "-" + i->second;
			}
			return ret;
		}
	};

	if (!dbCon)
		return -1;

	Query query(dbCon);
	query << 
		" select "
		"     TaskNumber, Name, TimeLimit, SpaceLimit "
		" from "
		"     task "
		" where "
		"     OlympId = " << OlympId << " and "
        "     OlympType = " << OlympType << " ";

	Result res(query.store());

	if (!query) return -2;
	if (res.rows() == 0) return 0;

	Result::const_iterator i = res.begin();
	Result::const_iterator e = res.end();
	const set< string > neededTasks = Helper::ParseNeededTasks(rrd_in);
	set< string >::const_iterator nde = neededTasks.end();
	set< string > obtainedTasks;
	map< string, string > timeLimits;
	map< string, string > spaceLimits;
	map< string, string > tasksNames;

	for(; i != e; ++i) {
		const string& taskNumber = (*i)[0];
		const string& taskName   = (*i)[1];
		const string& timeLimit  = (*i)[2];
		const string& spaceLimit = (*i)[3];
		if(neededTasks.empty() || neededTasks.find(taskNumber) != nde) {
			obtainedTasks.insert(taskNumber);
			timeLimits[taskNumber] = timeLimit;
			spaceLimits[taskNumber] = spaceLimit;
			tasksNames[taskNumber] = taskName;
		}
	}

	query.clear();
	query << 
		" select "
		"     td.TaskNumber, td.Solution, td.Grammar, "
		"     td.Filter, td.SCompilerId, td.FCompilerId "
//		"     td.Filter, sc.Name, fc.Name "
		" from "
		"     task_data as td, task as t "
//		"     task_data as td, task as c, compiler as sc, compiler as fc "
		" where "
		"     t.OlympId = " << OlympId << " and "
		"     t.OlympType = " << OlympType << " and "
		"     t.OlympId = td.OlympId and "
		"     t.OlympType = td.OlympType and"
		"     t.TaskNumber = td.TaskNumber ";
//		"     and "
//		"     td.SCompilerId = sc.CompilerId and "
//		"     td.FCompilerId = fc.CompilerId "
	res = query.store();

	if (!query) return -2;
	//if (res.rows() == 0) return 0;

	string body;
	TextHelper textHelper(body);
	i = res.begin();
	e = res.end();
	string text;
	int blockNumber = 0;

	for(; i != e; ++i) {
		const string& taskNumber    = (*i)[0];
		string solution      = (*i)[1];
		string grammar       = (*i)[2];
		string filter        = (*i)[3];
		const string& sCompilerId   = (*i)[4];
		const string& fCompilerId   = (*i)[5];
//		const string& sCompilerName = (*i)[4];
//		const string& fCompilerName = (*i)[5];
		
		if(neededTasks.empty() || neededTasks.find(taskNumber) != nde) {

			if(TextHelper::NotNull(solution) && TextHelper::NotNull(sCompilerId)) {
				aux_escapeString(solution);
				text = CodedBase64String(solution);
				textHelper.AddStringToText("__SOLUTION__");
				textHelper.AddHeaderToText("Task-Number", taskNumber);
				textHelper.AddHeaderToText("Compiler", sCompilerId);
//				textHelper.AddHeaderToText("Compiler", sCompilerName);
				textHelper.AddHeaderToText("Content-Length", text.size());
				textHelper.AddStringToText();
				textHelper.AddStringToText(text);
				textHelper.AddStringToText();
				++blockNumber;
			}

			if(TextHelper::NotNull(grammar)) {
				aux_escapeString(grammar);
				text = CodedBase64String(grammar);
				textHelper.AddStringToText("__GRAMMAR__");
				textHelper.AddHeaderToText("Task-Number", taskNumber);
				textHelper.AddHeaderToText("Content-Length", text.size());
				textHelper.AddStringToText();
				textHelper.AddStringToText(text);
				textHelper.AddStringToText();
				++blockNumber;
			}

			if(TextHelper::NotNull(filter) && TextHelper::NotNull(fCompilerId)) {
				aux_escapeString(filter);
				text = CodedBase64String(filter);
				textHelper.AddStringToText("__FILTER__");
				textHelper.AddHeaderToText("Task-Number", taskNumber);
				textHelper.AddHeaderToText("Compiler", fCompilerId);
//				textHelper.AddHeaderToText("Compiler", fCompilerName);
				textHelper.AddHeaderToText("Content-Length", text.size());
				textHelper.AddStringToText();
				textHelper.AddStringToText(text);
				textHelper.AddStringToText();
				++blockNumber;
			}
		}
	}

	query.clear();
	query << 
		" select "
		"     tst.TaskNumber, tst.TestNumber, tst.Input, tst.Output "
		" from "
		"     task as tsk, test as tst "
		" where "
		"     tsk.OlympId = " << OlympId << " and "
		"     tsk.OlympType = " << OlympType << " and "
		"     tsk.OlympId = tst.OlympId and "
		"     tsk.OlympType = tst.OlympType and"
		"     tsk.TaskNumber = tst.TaskNumber ";

	res = query.store();

	if (!query) return -2;
	//if (res.rows() == 0) return 0;

	i = res.begin();
	e = res.end();
	for(; i != e; ++i) {
		const string& taskNumber = (*i)[0];
		const string& testNumber = (*i)[1];
		string testText   = (*i)[2];
		string output     = (*i)[3];

		if(neededTasks.empty() || neededTasks.find(taskNumber) != nde) {

			aux_escapeString(testText);
			text = CodedBase64String(testText);
			textHelper.AddStringToText("__TEST__");
			textHelper.AddHeaderToText("Task-Number", taskNumber);
			textHelper.AddHeaderToText("Test-Number", testNumber);
			textHelper.AddHeaderToText("Content-Length", text.size());
			textHelper.AddStringToText();
			textHelper.AddStringToText(text);
			textHelper.AddStringToText();
			++blockNumber;

			aux_escapeString(output);
			text = CodedBase64String(output);
			textHelper.AddStringToText("__OUTPUT__");
			textHelper.AddHeaderToText("Task-Number", taskNumber);
			textHelper.AddHeaderToText("Test-Number", testNumber);
			textHelper.AddHeaderToText("Content-Length", text.size());
			textHelper.AddStringToText();
			textHelper.AddStringToText(text);
			textHelper.AddStringToText();
			++blockNumber;
		}
	}

	RRData::t_headers headers;
	char buf[64];
	if(neededTasks.empty()) {
		headers["Tasks-Number"] = itoa((int)obtainedTasks.size(), buf, 10);
	} else {
		headers["Tasks"] = Helper::ParseTaskString(obtainedTasks);
	}
	headers["Time-Limits"]    = Helper::ParseStdHeaderString(timeLimits);
	headers["Space-Limits"]   = Helper::ParseStdHeaderString(spaceLimits);
	headers["Tasks-Names"]    = Helper::ParseStdHeaderString(tasksNames);
	headers["Blocks-Number"]   = itoa(blockNumber, buf, 10);
	headers["Content-Length"] = itoa((int)body.size(), buf, 10);

	rrd_out.headers.swap(headers);
	rrd_out.body.swap(body);

	return 1;
}

int aux_escapeString(string& str){
	string tmp = "";
	string::size_type i = 0;
	while (i < str.size()){
		if (str[i] == '\n'){
			if ((i > 0) && (str[i - 1] != '\r')){
				tmp += "\r";
			} else {
				if (i == 0)
					tmp += "\r";
			}
		}
		tmp += str[i];
		i++;
	}
	str = tmp;
	return (int)str.size();
}

int aux_remake(int OlympId, int OlympType){
	if (!dbCon)
		return -1;

	Query query2(dbCon);
	query2 << "DELETE FROM rating WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType;
	if (!query2.exec())
		return -2;

	Query query(dbCon);
	query <<
		" select LogEntryId, TaskNumber, TeamId, ProgrammText, CompilerId, Penalty"
		" from log"
		" where OlympId=" << OlympId << " AND OlympType=" << OlympType;
	query << " order by LogEntryId";

	Result res = query.store();

	if (!query) return -2;
	if (res.rows() == 0) return 0;

	int Penalty;

	for (unsigned int i = 0; i < res.rows(); i++){
		Penalty = atoi(res[i]["Penalty"].c_str());
		testers.insertJob("-1", res[i]["TaskNumber"], res[i]["CompilerId"], res[i]["ProgrammText"], -1, res[i]["LogEntryId"], res[i]["TeamId"], Penalty, false);
	}

	return 1;
}

int aux_updateRating(int OlympId, int OlympType, map< int, t_ratingNode >& data){
	if (!dbCon)
		return -1;

	Query query(dbCon);
	query << "SELECT a.Name as Name, a.TeamId as TeamId, b.TaskNumber as TaskNumber, b.Result as Result, b.FailsNumber as FailsNumber, b.Penalty as Penalty";
	query << " FROM team as a, rating as b";
	query << " WHERE b.OlympId=" << OlympId << " AND b.OlympType=" << OlympType << " AND";
	query << " a.TeamId=b.TeamId order by a.Name";
	
	Result res = query.store();
	if (!query) return -2;
	if (res.rows() == 0) return 0;

	data.clear();
	
	map<int, t_ratingNode>::iterator user;
	int TeamId, TaskNumber, FailsNumber, Penalty;
	char Result;
	for (unsigned int i = 0; i < res.rows(); i++){
		TeamId = atoi(res[i]["TeamId"].c_str());
		TaskNumber = atoi(res[i]["TaskNumber"].c_str());
		FailsNumber = atoi(res[i]["FailsNumber"].c_str());
		Penalty = atoi(res[i]["Penalty"].c_str());
		Result = res[i]["Result"][0];
		if ((user = data.find(TeamId)) == data.end()){
			data.insert(pair< int, t_ratingNode >(TeamId, t_ratingNode(TeamId, res[i]["Name"])));
			if ((user = data.find(TeamId)) == data.end())
				return -3;
		}
		user->second.tasks.insert(pair< int, t_taskData >(TaskNumber, t_taskData(TaskNumber, Result, FailsNumber, Penalty)));
		if (Result == '+'){
			user->second.SolvedTasks = user->second.SolvedTasks + 1;
			user->second.OverallPenalty = user->second.OverallPenalty + 1200 * FailsNumber + Penalty;
		}
	}
	return 1;
}

int aux_getLastLogId(int OlympId, int OlympType){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT MAX(LogEntryId) AS maxid FROM log";
	q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType;

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	int num = atoi(res[0]["maxid"].c_str());

	return num;
}

int aux_getTeams(int OlympId, vector< pair< int, string > >& teams){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT TeamId, Name, Status FROM team";
	q << " WHERE OlympId=" << OlympId;

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	for (unsigned int i = 0; i < res.rows(); i++){
		teams.push_back(pair< int, string >(atoi(res[i]["TeamId"].c_str()), res[i]["Name"]));
	}

	return (int)teams.size();
}

int aux_getTasks(int OlympId, int OlympType, vector< int >& tasks){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT TaskNumber FROM task";
	q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType;
	q << " ORDER BY TaskNumber";

	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	for (unsigned int i = 0; i < res.rows(); i++){
		tasks.push_back(atoi(res[i]["TaskNumber"].c_str()));
	}

	return (int)tasks.size();
}

int aux_getPartOfRating(int OlympId, int OlympType, vector< t_ratingData >& rating, int From, int To){
	if (!dbCon) return -1;

	Query q = dbCon;
	q << "SELECT LogEntryId, TeamId, TaskNumber, Result, Penalty FROM log";
	q << " WHERE OlympId=" << OlympId << " AND OlympType=" << OlympType;
	q << " AND LogEntryId>" << From;
	if (To >= 0)
		q << " AND LogEntryId<=" << To;
	q << " ORDER BY LogEntryId";


	Result res = q.store("NULL");
	if (!q) return -2;
	if (res.rows() == 0) return 0;

	t_ratingData tmp;
	
	int LastId = 0;
	
	for (unsigned int i = 0; i < res.rows(); i++){
		tmp.Penalty = atoi(res[i]["Penalty"].c_str());
		tmp.Result = atoi(res[i]["Result"].c_str());
		tmp.TaskNumber = atoi(res[i]["TaskNumber"].c_str());
		tmp.TeamId = atoi(res[i]["TeamId"].c_str());
		rating.push_back(tmp);
		LastId = atoi(res[i]["LogEntryId"].c_str());
	}
	
	return LastId;
}

int aux_disqualifyTeam(int DBId, bool set){
	if (!dbCon) return -1;

	Query q = dbCon;
	if (set)
		q << "UPDATE team SET Status=\'D\'";
	else
		q << "UPDATE team SET Status=\'N\'";
	q << " WHERE TeamId=" << DBId;

	Result res = q.store("NULL");
	if (!q.exec()) return -2;
	if (res.rows() == 0) return 0;

	return 1;
}