#include "handlers.h"

void defaultHandler(const RRData& in, RRData& out){
	// check command line length
	if (in.command.name.size() >= MAX_LINE_SIZE){
		res_404(in, out, "Command line too long");
		return;
	}

	// check protocol name
	if (stricmp(in.protocol.name.c_str(), "OLYMPIA") != 0){
		res_404(in, out, "Bad protocol name");
		return;
	}

	// check protocol version
	if ((in.protocol.version.major != 1) || (in.protocol.version.minor != 0)){
		res_502(in, out);
		return;
	}

	// check number of headers
	if (in.headers.size() >= MAX_HEADERS_COUNT){
		res_404(in, out, "Too much headers");
		return;
	}

	// check Content-Length existence
	if ((in.headers.find("Content-Length") != in.headers.end()) && (in.headers.find("Content-Length")->second.compare("-1") == 0)){
		res_403(in, out);
		return;
	}

	res_404(in, out);
}