#include "handlers.h"

void req_DONE(const RRData& in, RRData& out){
	if (isUnknown(in.params.param1)){
		res_400(in, out);
		return;
	}

	if (!isTester(in.params.param1)){
		res_401(in, out);
		return;
	}

	Tester::t_testerInfo* info = (Tester::t_testerInfo*)testers.getClientInfo(in.params.param2);
	if (info == NULL){
		res_500(in, out, "Fatal error: diamond array");
		return;
	}

	//   DONE  ,      
	if (info->serveTime == 0){
		unknown.insertClient(info->protocol, info->IP.c_str());
		info->deleted = true;
		res_404(in, out, "Request DONE is illegal because no task was appointed to tester. Channel is closed.");
		return;
	}

	if (in.headers.find("Result") == in.headers.end()){
		res_404(in, out, "Obligatory header is missing");
		info->serveTime = 0;
		if (info->checkQueue.empty())
			testers.insertJob(info->job);
		return;
	}

	string result = in.headers.find("Result")->second;
	string testNumber = "";
	if (in.headers.find("Test-Number") != in.headers.end())
		testNumber = in.headers.find("Test-Number")->second;

	int intResult = atoi(result.c_str());
	int intTestNumber = atoi(testNumber.c_str());
	if (result.compare("0") != 0){
		if (intResult == 0 || intResult > 6 || intResult < -2){
			res_404(in, out, "Bad result value");
			info->serveTime = 0;
			if (info->checkQueue.empty())
				testers.insertJob(info->job);
			return;
		}
	}
	
	int tests;
	if (!testNumber.empty()){
		if (info->checkQueue.empty())
			tests = aux_getTestsNumber(info->job.Task, config.olympiad.OlympId, config.olympiad.OlympType);
		else
			tests = aux_getTestsNumber(info->checkQueue.front().Task, config.olympiad.OlympId, config.olympiad.OlympType);
	
		if (tests == -1){
			res_500(in, out, "Database is not available");
			info->serveTime = 0;
			if (info->checkQueue.empty())
				testers.insertJob(info->job);
			return;
		}
		if (tests == -2){
			res_500(in, out, "Database error");
			info->serveTime = 0;
			if (info->checkQueue.empty())
				testers.insertJob(info->job);
			return;
		}
			
		if (intTestNumber <= 0 || intTestNumber > tests){
			res_404(in, out, "Bad test number");
			info->serveTime = 0;
			if (info->checkQueue.empty())
				testers.insertJob(info->job);
			return;
		}
	}

	if ((intResult == -1) || (intResult == -2)){
		unknown.insertClient(info->protocol, info->IP.c_str());
		info->deleted = true;
		res_201(in, out);
		info->ready = false;
		info->serveTime = 0;
		if (info->checkQueue.empty())
			testers.insertJob(info->job);
		return;
	}

	if ((intResult >= 2) && (intResult <=6) && (intTestNumber == 0)){
		res_404(in, out, "Result value implies test number.");
		info->serveTime = 0;
		if (info->checkQueue.empty())
			testers.insertJob(info->job);
		return;
	}

	if (!info->checkQueue.empty()){
		if ((info->checkQueue.front().expectedResult.result.compare(result) != 0) ||
			(info->checkQueue.front().expectedResult.testNumber.compare(testNumber) != 0)){
				unknown.insertClient(info->protocol, info->IP.c_str());
				info->deleted = true;
				info->serveTime = 0;
				res_406(in, out);
				return;
			} else {
				info->checkQueue.pop();
				info->serveTime = 0;
				res_204(in, out);
				return;
			}
	}
	
	if (testNumber.empty()) testNumber = "0";
	int tmp = aux_registerResultOfTesting(info->job.DBId, result, testNumber, config.olympiad.OlympId, config.olympiad.OlympType, info->job.Task, info->job.TeamId, info->job.Penalty);
	config.rating.actual = false;
	
	if (tmp == -1){
		res_500(in, out, "Database is not available");
		info->serveTime = 0;
		testers.insertJob(info->job);
		return;
	}
	if (tmp == -2){
		res_500(in, out, "Database error");
		info->serveTime = 0;
		testers.insertJob(info->job);
		return;
	}
	
	info->serveTime = 0;
	if ((info->job.clientId != -1) && (info->job.TaskId.compare("-1") != 0))
		users.msg_resultOfTesting(info->job.clientId, info->job.TaskId, result, testNumber);

	int res;
	if (info->reactivated == false){
		res = aux_getCheckTasks(info->checkQueue, config.olympiad.OlympId, config.olympiad.OlympType);
		if (!DBCheck(res, in, out)) return;
		
		info->ready = false;
		info->serveTime = 0;
		info->reactivated = true;
	}

	res_204(in, out);
}