#include "handlers.h"

void req_LOGIN(const RRData& in, RRData& out){
	// if the request has no parameter
	if (in.command.parameter.empty()){
		res_404(in, out, "Obligatory parameter is missing");
		return;
	}

	// checking if channel is already opened
	if (!isUnknown(in.params.param1)){
		res_404(in, out, "Channel is already opened");
		return;
	}

	UNknown::t_unknownInfo* info = (UNknown::t_unknownInfo*)unknown.getClientInfo(in.params.param2);
	if (info == NULL){
		res_500(in, out, "Fatal error: orange melon");
		return;
	}
	
	// if it is user
	if (stricmp(in.command.parameter.c_str(), "team") == 0){
		// checking IP
		if (users.CheckIP(info->IP) == false){
			res_400(in, out, "Access from this IP address is forbidden");
			return;
		}

		// checking code and disqualification flag
		RRData::t_headers::const_iterator i;
		if ((i = in.headers.find("Password")) == in.headers.end()){
			res_404(in, out, "Obligatory header is missing");
			return;
		}

		if (testers.Size() == 0){
			res_500(in, out, "No testers available");
			return;
		}

		string dbId;
		int res = aux_checkCode(i->second, info->IP, dbId, config.olympiad.OlympId);
		if (res ==  0){	res_405(in, out); return; }
		if (res == -1){	res_500(in, out, "Database is not available"); return; }
		if (res == -2){ res_500(in, out, "Database error");	return; }
		if (res == -3){	res_402(in, out); return; }
		
		if (config.olympiad.started == true)
			res_209(in, out);
		else
			res_100(in, out);
		
		info->deleted = true;
		int newId = users.insertClient(info->protocol, info->IP.c_str());
		User::t_userInfo* info = (User::t_userInfo*)users.getClientInfo(newId);
		if (info == NULL){
			res_500(in, out, "Fatal error: exercise book for the copies");
			return;
		}
		info->DBId = dbId;
		info->disqualified = false;
		
		return;
	}

	// if it is tester
	if (stricmp(in.command.parameter.c_str(), "tester") == 0){
		// checking IP
		if (testers.CheckIP(info->IP) == false){
			res_400(in, out, "Access from this IP address is forbidden");
			return;
		}
		
		RRData::t_headers::const_iterator i;
		if ((i = in.headers.find("Compilers")) == in.headers.end()){
			res_404(in, out, "Obligatory header is missing");
			return;
		}
		
		string compilers = i->second;
		set< string > SupportedCompilers;
		
		parseSemicolonString(SupportedCompilers, compilers);
		int res = aux_checkCompilersLists(SupportedCompilers, config.olympiad.OlympId);
		if (res ==  0){	res_400(in, out, "Not all compilers are supported"); return; }
		if (res == -1){	res_500(in, out, "Database is not available"); return; }
		if (res == -2){	res_500(in, out, "Database error"); return; }
		
		res_200(in, out);
		info->deleted = true;
		int newId = testers.insertClient(info->protocol, info->IP.c_str());
		Tester::t_testerInfo* info = (Tester::t_testerInfo*)testers.getClientInfo(newId);
		if (info == NULL){
			res_500(in, out, "Fatal error: civil service");
			return;
		}
		
		// filling check tasks
		res = aux_getCheckTasks(info->checkQueue, config.olympiad.OlympId, config.olympiad.OlympType);
		if (res == -1){ res_500(in, out, "Database is not available"); return; }
		if (res == 0){ res_500(in, out, "Database error"); return; }
		
		info->ready = false;
		info->serveTime = 0;

		return;
	}

	// if it is admin
	if (stricmp(in.command.parameter.c_str(), "admin") == 0){
		// checking IP
		if (admins.CheckIP(info->IP) == false){
			res_400(in, out, "Access from this IP address is forbidden");
			return;
		}
		
		// checking password
		RRData::t_headers::const_iterator i;
		if ((i = in.headers.find("Password")) == in.headers.end()){
			res_404(in, out, "Obligatory header is missing");
			return;
		}
		string dbId;
		int res = aux_checkPassword(i->second);
		if (res ==  0){	res_400(in, out, "Wrong password"); return; }
		if (res == -1){	res_500(in, out, "Database is not available"); return; }
		if (res == -2){ res_500(in, out, "Database error");	return; }
		
		res_200(in, out);
		
		info->deleted = true;
		admins.insertClient(info->protocol, info->IP.c_str());
		
		return;
	}
	
	// todo:    rating

	res_404(in, out, "Wrong parameter value");
}