#include "handlers.h"

//////////////////////////////////////////////////////////////////
// Response handlers
//////////////////////////////////////////////////////////////////

void res_100(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "100";
	out.command.parameter = "Wait For Beginning";
	out.headers.clear();
	out.body = "";
}

void res_101(const RRData& in, RRData& out, const string& TaskId){
	makeCommandHeader(out.protocol);
	out.command.name = "101";
	out.command.parameter = "Program Accepted For Testing";
	out.headers.clear();
	out.headers.insert(RRData::t_header("Task-Id", TaskId));
	out.body = "";
}

void res_102(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "102";
	out.command.parameter = "Free Tester Registered";
	out.headers.clear();
	out.body = "";
}

void res_200(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "200";
	out.command.parameter = "Logged In";
	out.headers.clear();
	out.body = "";
}

void res_201(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "201";
	out.command.parameter = "Logged Out";
	out.headers.clear();
	out.body = "";
}

void res_202(const RRData& in, RRData& out, const string& TaskId, const string& Result, const string& TestNumber){
	makeCommandHeader(out.protocol);
	out.command.name = "202";
	out.command.parameter = "Result Of Testing";
	out.headers.clear();
	out.headers.insert(RRData::t_header("Task-Id", TaskId));
	out.headers.insert(RRData::t_header("Result", Result));
	if (TestNumber.compare("0") != 0)
		out.headers.insert(RRData::t_header("Test-Number", TestNumber));
	out.body = "";
}

void res_203(const RRData& in, RRData& out, int OlympId, int OlympType, int& res){
	makeCommandHeader(out.protocol);
	out.command.name = "203";
	out.command.parameter = "Tests And Tasks";
	out.headers.clear();
	
	res = aux_form203Respose(in, out, OlympId, OlympType);
}

void res_204(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "204";
	out.command.parameter = "Result Accepted";
	out.headers.clear();
	out.body = "";
}

void res_205(const RRData& in, RRData& out, const char* mes){
	makeCommandHeader(out.protocol);
	out.command.name = "205";
	out.command.parameter = "OK";
	out.headers.clear();
	out.headers.insert(RRData::t_header("Message", mes));
	out.body = "";
}

int compare(const void* arg1, const void* arg2){
	struct t_teamData{
		t_teamData(int _TeamId, const string& _TeamName, map< int, t_ratingNode >* _Data) { TeamId = _TeamId; TeamName = _TeamName; Data = _Data; }
		int TeamId;
		string TeamName;
		map< int, t_ratingNode >* Data;
	};

	t_teamData* a1 = *((t_teamData**)arg1);
	t_teamData* a2 = *((t_teamData**)arg2);
	int solvedTasks1, solvedTasks2;
	int penalty1, penalty2;
	map< int, t_ratingNode >::iterator i;
	if ((i = a1->Data->find(a1->TeamId)) == a1->Data->end()){
		solvedTasks1 = 0;
		penalty1 = 0;
	} else {
		solvedTasks1 = i->second.SolvedTasks;
		penalty1 = i->second.OverallPenalty;
	}
	if ((i = a2->Data->find(a2->TeamId)) == a2->Data->end()){
		solvedTasks2 = 0;
		penalty2 = 0;
	} else {
		solvedTasks2 = i->second.SolvedTasks;
		penalty2 = i->second.OverallPenalty;
	}
	if (solvedTasks1 < solvedTasks2) return 1;
	if (solvedTasks1 > solvedTasks2) return -1;
	if (penalty1 < penalty2) return -1;
	if (penalty1 > penalty2) return 1;
	return 0;
}

void res_206(const RRData& in, RRData& out, map< int, t_ratingNode >& data, int lastId){
	makeCommandHeader(out.protocol);
	out.command.name = "206";
	out.command.parameter = "Full Rating";
	out.headers.clear();

	struct t_teamData{
		t_teamData(int _TeamId, const string& _TeamName, map< int, t_ratingNode >* _Data) { TeamId = _TeamId; TeamName = _TeamName; Data = _Data; }
		int TeamId;
		string TeamName;
		map< int, t_ratingNode >* Data;
	};
	vector< pair< int, string > > tmpTeams;
	vector< int > tasks;
	t_teamData** teams;
	
	// make 'teams' table with all teams
	int res = aux_getTeams(config.olympiad.OlympId, tmpTeams);
	if (!DBCheck(res, in, out)) return;
	if (res == 0){ res_500(in, out, "No commands available in database"); return; }
	teams = new t_teamData*[res];
	for (int i = 0; i < res; i++)
		teams[i] = new t_teamData(tmpTeams[i].first, tmpTeams[i].second, &data);

	// make tasks table with all tasks
	res = aux_getTasks(config.olympiad.OlympId, config.olympiad.OlympType, tasks);
	if (!DBCheck(res, in, out)) return;
	if (res == 0){ res_500(in, out, "No tasks available in database"); return; }

	// sort teams table. Sorting order depends on what is stored in data
	qsort((void*)teams, tmpTeams.size(), sizeof(t_teamData*), compare);

	// make headers
	char b[100];
	out.headers.insert(RRData::t_header("Teams-Number", itoa((int)tmpTeams.size(), b, 10)));
	out.headers.insert(RRData::t_header("Tasks-Number", itoa((int)tasks.size(), b, 10)));
	if (lastId >= 0)
		out.headers.insert(RRData::t_header("Last-Id", itoa(lastId, b, 10)));
	
	// make output
	out.body = "";
	map< int, t_taskData >::iterator p;
	map< int, t_ratingNode >::iterator s;
	for (unsigned int i = 0; i < tmpTeams.size(); i++){
		out.body += itoa(teams[i]->TeamId, b, 10); out.body += "\t";
		out.body += teams[i]->TeamName; out.body += "\t";
		s = teams[i]->Data->find(teams[i]->TeamId);
		for (unsigned int j = 0; j < tasks.size(); j++){
			if ((s != teams[i]->Data->end()) && ((p = s->second.tasks.find(tasks[j])) != s->second.tasks.end())){
				out.body += p->second.Result;
				if (p->second.FailsNumber > 0)
					out.body += itoa(p->second.FailsNumber, b, 10);
			}
			out.body += "\t";
		}
		if (s != teams[i]->Data->end()){
			out.body += itoa((int)s->second.SolvedTasks, b, 10); out.body += "\t";
			out.body += itoa((int)s->second.OverallPenalty, b, 10); out.body += "\n";
		} else
			out.body += "0\t0\n";
	}

	for (unsigned int i = 0; i < tmpTeams.size(); i++)
		delete teams[i];
	delete[] teams;
}

void res_207(const RRData& in, RRData& out, int From, int To){
	makeCommandHeader(out.protocol);
	out.command.name = "207";
	out.command.parameter = "Part Of Rating";

	vector< t_ratingData > rating;
	int res;
	if (To >= 0)
		res = aux_getPartOfRating(config.olympiad.OlympId, config.olympiad.OlympType, rating, From, To);
	else
		res = aux_getPartOfRating(config.olympiad.OlympId, config.olympiad.OlympType, rating, From);
	if (!DBCheck(res, in, out)) return;
	if (res == 0){
		res_208(in, out, From);
		return;
	}

	out.body = "";
	char buf[100];
	for (unsigned int i = 0; i < rating.size(); i++){
		out.body += itoa(rating[i].TeamId, buf, 10); out.body += "\t";
		out.body += itoa(rating[i].TaskNumber, buf, 10); out.body += "\t";
		out.body += itoa(rating[i].Result, buf, 10); out.body += "\t";
		out.body += itoa(rating[i].Penalty, buf, 10); out.body += "\n";
	}
	out.headers.clear();
	out.headers.insert(RRData::t_header("From", itoa(res, buf, 10)));
	out.headers.insert(RRData::t_header("Records-Number", itoa((int)rating.size(), buf, 10)));
}

void res_208(const RRData& in, RRData& out, int From){
	makeCommandHeader(out.protocol);
	out.command.name = "208";
	out.command.parameter = "Rating Not Changed";
	out.headers.clear();
	char buf[100];
	out.headers.insert(RRData::t_header("From", itoa(From, buf, 10)));
	out.body = "";
}

void res_209(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "209";
	out.command.parameter = "Olympiad Started";
	out.headers.clear();
	out.body = "";
}

void res_210(const RRData& in, RRData& out, const string& TaskId){
	makeCommandHeader(out.protocol);
	out.command.name = "210";
	out.command.parameter = "Already Solved";
	out.headers.clear();
	out.headers.insert(RRData::t_header("Task-Id", TaskId));
	out.body = "";
}

void res_211(const RRData& in, RRData& out, int OlympId, int OlympType, int& res){
	makeCommandHeader(out.protocol);
	out.command.name = "211";
	out.command.parameter = "Tasks";
	out.headers.clear();
	
	res = aux_form211Respose(out, OlympId, OlympType);
}

void res_212(const RRData& in, RRData& out, int OlympId, int OlympType, int& res){
	makeCommandHeader(out.protocol);
	out.command.name = "212";
	out.command.parameter = "Compilers";
	out.headers.clear();
	
	res = aux_form212Respose(out, OlympId, OlympType);
}

void res_214(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "214";
	out.command.parameter = "Olympiad Stopped";
	out.headers.clear();
}

void res_300(const RRData& in, RRData& out, const string& tasks){
	makeCommandHeader(out.protocol);
	out.command.name = "300";
	out.command.parameter = "Reactivate Tests And Tasks";
	out.headers.clear();
	if (!tasks.empty())
		out.headers.insert(RRData::t_header("Tasks", tasks));
	out.body.clear();
}

void res_301(const RRData& in, RRData& out, const string& Task, const string& Compiler, const string& ProgramText, bool coded){
	makeCommandHeader(out.protocol);
	out.command.name = "301";
	out.command.parameter = "Program For Testing";
	out.headers.clear();
	out.headers.insert(RRData::t_header("Task", Task));
	out.headers.insert(RRData::t_header("Compiler", Compiler));
	
	string body;
	char buf[256];
	if (!coded) {
		body = CodedBase64String(ProgramText);
		itoa((int)body.size(), (char*)buf, 10);
		out.headers.insert(RRData::t_header("Content-Length", buf));
		out.body = body;
	} else {
		itoa((int)ProgramText.size(), (char*)buf, 10);
		out.headers.insert(RRData::t_header("Content-Length", buf));
		out.body = ProgramText;
	}
}

void res_400(const RRData& in, RRData& out, const char* mes){
	makeCommandHeader(out.protocol);
	out.command.name = "400";
	out.command.parameter = "Forbidden";
	out.headers.clear();
	if (mes != NULL)
		out.headers.insert(RRData::t_header("Message", mes));
	out.body = "";
}

void res_401(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "401";
	out.command.parameter = "Method Not Allowed";
	out.headers.clear();
	out.body = "";
}

void res_402(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "402";
	out.command.parameter = "Team Disqualified";
	out.headers.clear();
	out.body = "";
}

void res_403(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "403";
	out.command.parameter = "Content-Length Required";
	out.headers.clear();
	out.body = "";
}

void res_404(const RRData& in, RRData& out, const char* mes){
	makeCommandHeader(out.protocol);
	out.command.name = "404";
	out.command.parameter = "Bad Request";
	out.headers.clear();
	out.headers.insert(RRData::t_header("Message", mes));
	out.body = "";
}

void res_405(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "405";
	out.command.parameter = "Code-Team Disparity";
	out.headers.clear();
	out.body = "";
}

void res_406(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "406";
	out.command.parameter = "Comparison Standard Disparity";
	out.headers.clear();
	out.body = "";
}

void res_407(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "407";
	out.command.parameter = "Olympiad Currently Running";
	out.headers.clear();
	out.body = "";
}

void res_409(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "409";
	out.command.parameter = "Olympiad Not Started";
	out.headers.clear();
	out.body = "";
}

void res_500(const RRData& in, RRData& out, const char* mes){
	makeCommandHeader(out.protocol);
	out.command.name = "500";
	out.command.parameter = "Internal Server Error";
	out.headers.clear();
	if (mes != NULL)
		out.headers.insert(RRData::t_header("Message", mes));
	out.body = "";
}

void res_502(const RRData& in, RRData& out){
	makeCommandHeader(out.protocol);
	out.command.name = "502";
	out.command.parameter = "OLYMPIA Version Not Supported";
	out.headers.clear();
	out.body = "";
}