#include "core.h"
#include "pr_server.h"
#include "pr_socket.h"

#include "handler.h"
#include "handlers.h"
#include "protocol.h"
#include "config.h"
#include "mysqlplus.h"
#include <iostream>
#include <time.h>

#include "user.h"
#include "unknown.h"

using namespace MySQLPlus;

using std::cerr;
using std::endl;

// Server configuration
Configuration config;

// Map of handlers
t_handlers handlerMap;
// Handler class that is responsible for current version of protocol
Handler handler(&handlerMap, defaultHandler, "OLYMPIA", 1, 0);

// Database connections
Connection dbCon("root", "root", "olympia", "localhost");
Connection dbAdminCon("root", "root", "mysql", "localhost");

// Client classes
User users;
Tester testers;
Admin admins;
UNknown unknown;

int main(int argc, char** argv){
	if (!dbCon || !dbAdminCon){
		cerr << "Error: can not connect to database" << endl;
		if (!dbCon)
			cerr << dbCon.error() << endl;
		if (!dbAdminCon)
			cerr << dbAdminCon.error() << endl;
		return 0;
	}
	// Making map of handlers
	formHandlerMap(handlerMap);

	// Opening accept socket for reading
	acceptSocket acceptSock(2006);
	if(!acceptSock.Created()){
		cout << pr_strerror(PR_ERRNO) << endl;
		return 0;
	}
	acceptSock.makeNonBlocking();
	stdSocket* s;

	// Infinite loop of serving requests
	while (1){
		// Listening for new arrivals
		s = acceptSock.Accept();
		if (s != NULL){
			// Checking if this user is  worth to live
			if (unknown.CheckIP(s->GetRemoteIp())){
				cout << "User on " << s->GetRemoteIp() << " added" << endl;
				unknown.insertClient(new ServerProtocol(s, &handler), s->GetRemoteIp());
			}
			else {
				// Wrong IP
				s->Close();
				delete s;
				cout << "User on " << s->GetRemoteIp() << " rejected" << endl;
			}
		}

		// Handling users
		unknown.Handle();
		unknown.RemoveTimeoutedClients();

		users.Handle();
		testers.Handle();
		admins.Handle();

		// Checking all kinds of things
		
		// If olympiad is over
		if (config.olympiad.started && (time(0) - config.olympiad.StartTime >= config.olympiad.Duration)){
			config.olympiad.started = false;
			config.rating.frozen = true;
			users.msg_olympiadStopped();
		}
		// If rating should be frozen
		if (!config.rating.frozen && (config.olympiad.started && (config.olympiad.StartTime + config.olympiad.Duration - time(0) <= config.olympiad.RatingFreeze))){
			config.rating.frozen = true;
			config.rating.LastIdBeforeFreezing = aux_getLastLogId(config.olympiad.OlympId, config.olympiad.OlympType);
			if (config.rating.LastIdBeforeFreezing < 0) config.rating.LastIdBeforeFreezing = 0;
			aux_updateRating(config.olympiad.OlympId, config.olympiad.OlympType, config.rating.data);
		}

		// Sleeping a little bit
		Sleep(1000);
	}

	return 0;
}