#include "tester.h"
#include "mysqlplus.h"
#include <iostream>
#include "handlers.h"


using namespace MySQLPlus;

extern Connection dbCon;

using std::cout;
using std::endl;

Tester::Tester(){
	Reload();
}

void Tester::Reload(void){
	IPs.clear();
	//     IP 
	Query q = dbCon;
	q << "SELECT b.IP as ip FROM host as a, ip as b WHERE a.Type=\'T\'";
	q << " AND a.IPId=b.IPId";

	Result res = q.store("NULL");
	if (!q || res.rows() == 0) return;
	for (unsigned int i = 0; i < res.rows(); i++){
		string ip = res[i]["IP"];
		if (!ip.empty())
			IPs.insert(ip);
	}
}

Client::t_info* Tester::putInfo(Protocol* protocol, const char* IP){
	return (t_info*)new t_testerInfo(protocol, IP);
}

int Tester::getCheckedTestersNumber(void){
	t_map::const_iterator i = clients.begin();
	t_map::const_iterator e = clients.end();

	int res = 0;
	while (i != e){
		if (i->second != NULL){
			if (((t_testerInfo*)(i->second))->checkQueue.empty())
				res++;
		}
		i++;
	}
	
	return res;
}

int Tester::getReadyTestersNumber(void){
	t_map::const_iterator i = clients.begin();
	t_map::const_iterator e = clients.end();

	int res = 0;
	while (i != e){
		if (i->second != NULL){
			if (((t_testerInfo*)(i->second))->ready)
				res++;
		}
		i++;
	}
	
	return res;
}

Tester::t_testerInfo* Tester::getReadyTester(void){
	t_map::const_iterator i = clients.begin();
	t_map::const_iterator e = clients.end();

	int res = 0;
	while (i != e){
		if (i->second != NULL){
			if (((t_testerInfo*)(i->second))->ready)
				return (t_testerInfo*)i->second;
		}
		i++;
	}
	
	return NULL;
}

void Tester::makeReady(unsigned int testerId, RRData& out){
	t_testerInfo* t = (t_testerInfo*)getClientInfo(testerId);
	
	if (t == NULL){
		res_500(out, out);
		return;
	}

	int res;
	if (t->reactivated == false){
		t->checkQueue.empty();
		res = aux_getCheckTasks(t->checkQueue, config.olympiad.OlympId, config.olympiad.OlympType);
		if (!DBCheck(res, out, out)) return;
		
		t->ready = false;
		t->serveTime = 0;
		t->reactivated = true;
	}

	if (jobs.empty()){
		t->ready = true;
		t->serveTime = 0;
		res_102(out, out);
		return;
	}

	if (t->protocol->Online()){
		t->job = jobs.front();
		t->ready = false;
		t->serveTime = time(0);
		res_301(out, out, jobs.front().Task, jobs.front().Compiler, jobs.front().ProgramText, jobs.front().coded);
		jobs.pop();
	}
}

void Tester::Reactivate(const string& tasks){
	t_map::iterator i = clients.begin();
	
	RRData out;
	res_300(out, out, tasks);

	while (i != clients.end()){
		if ((i->second->protocol != NULL) && (i->second->protocol->Online())){
			i->second->protocol->Generate(out);
		}
		((t_testerInfo*)(i->second))->reactivated = false;
		i++;
	}
}

void Tester::insertJob(const string& TaskId, const string& Task, const string& Compiler, const string& ProgramText, int clientId, const string DBId, const string& TeamId, unsigned int Penalty, bool coded){
	jobs.push(t_job(Task, Compiler, DBId, clientId, TaskId, ProgramText, TeamId, Penalty, coded));

	appointJobs();
}

void Tester::insertJob(const t_job& job){
	jobs.push(job);

	appointJobs();
}

void Tester::appointJobs(void){
	t_testerInfo* info;
	RRData out;

	while (1){
		if (getReadyTestersNumber() <= 0) break;
		if (jobs.empty()) break;
		
		info = getReadyTester();
		if (info == NULL) break;
		info->job = jobs.front();
		jobs.pop();
		info->ready = false;
		info->serveTime = time(0);
		res_301(out, out, info->job.Task, info->job.Compiler, info->job.ProgramText, info->job.coded);
		if (info->protocol->Online())
			info->protocol->Generate(out);
	}
}